#include "palm1lib.c"
#include "p_apps.c"

#ifdef AUTO_TEST

void SdramFullTest();

void AutoTestLcd()
{
	PXCHAR pBuf;
	XCHAR i;
	
	DisplayString((PCCHAR)"Start LCD Test", 0);
	DisplayString((PCCHAR)"", 1);

	SysSleep(1000000);

	for (i = 0; i < DISPLAY_MAX_CHAR; i ++)
	{
		pBuf[i] = 0xff;
	}
	DisplayLine(pBuf, 0);
	DisplayLine(pBuf, 1);

	SysSleep(1000000);

	DisplayCursor(TRUE);

	SysSleep(1000000);

	for (i = 0; i < DISPLAY_MAX_CHAR; i ++)
	{
		pBuf[i] = 'A';
	}
	DisplayLine(pBuf, 0);
	DisplayLine(pBuf, 1);

	SysSleep(1000000);
}

void AutoTestLed()
{
	XCHAR i, j;

	DisplayString((PCCHAR)"Start LED Test", 0);
	DisplayString((PCCHAR)"", 1);

	for (i = 0; i < 2; i ++)
	{
		Led_On(NORMAL_LIGHT);
		Led_On(SECOND_LIGHT);
		Led_On(KEYPAD_LIGHT);
		for (j = 0; j < 4; j ++)
		{
			Cam_LedOn(i);
		}

		SysSleep(500000);

		Led_Off(NORMAL_LIGHT);
		Led_Off(SECOND_LIGHT);
		Led_Off(KEYPAD_LIGHT);
		for (j = 0; j < 4; j ++)
		{
			Cam_LedOff(i);
		}

		SysSleep(500000);
	}
}

void AutoTestSdram()
{
	DisplayString((PCCHAR)"Start Sdram Test", 0);
	DisplayString((PCCHAR)"", 1);

	SysSleep(1000000);
	
	SdramFullTest();
}

void AutoTestKeypad()
{
}

void AutoTestAc97()
{
}

#endif